import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';

class MockWalletApi extends Mock implements WalletApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eWalletService _m4eWalletService;

  MockWalletApi _mockWalletApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockWalletApi = MockWalletApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eWalletService = M4eWalletService(
      walletApi: _mockWalletApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _userId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _tWallets = <M4eWallet>[
    M4eWallet(
      id: M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0'),
      walletInfo: M4eWalletInfo(label: 'W1', currency: 'GHS'),
    ),
    M4eWallet(
      id: M4eUniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88'),
      walletInfo: M4eWalletInfo(label: 'W2', currency: 'GHS'),
    ),
  ];

  group('m4e-wallet-test | ', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockWalletApi.fetchUserWallets(any))
          .thenAnswer((_) async => _tWallets);

      // act
      await _m4eWalletService.getUserWallets(_userId);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
        when(_mockWalletApi.fetchUserWallets(any))
            .thenAnswer((_) async => _tWallets);
      });

      test('should return List<Wallet> on success call', () async {
        // arrange
        when(_mockWalletApi.fetchUserWallets(any))
            .thenAnswer((_) async => _tWallets);

        // act
        final _result = await _m4eWalletService.getUserWallets(_userId);

        // assert
        expect(_result, equals(_tWallets));
      });

      test('should re-throw a ServerException when server call fails',
          () async {
        // arrange
        when(_mockWalletApi.fetchUserWallets(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eWalletService.getUserWallets;

        // assert
        expect(
            () => _call(_userId), throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });
      test(
          'should throw [ServerException] with [NO_INTERNET_CONNECTION] error code',
          () async {
        // arrange
        when(_mockWalletApi.fetchUserWallets(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eWalletService.getUserWallets;

        // assert
        expect(
            () => _call(_userId), throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });
}
